<?php
// Copyright 1999-2024. WebPros International GmbH. All rights reserved.

namespace WHMCS\Module\Server\SolusVm2Vps\WhmcsAPI;

use Exception;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\ConfigurableOptionsValues;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\Hosting;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\Product as ProductModel;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\ProductConfigOption;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\Server;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\SolusServer;
use WHMCS\Module\Server\SolusVm2Vps\Database\Models\Upgrade;
use WHMCS\Module\Server\SolusVm2Vps\Helpers\Arr;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Connector;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Requests\ServerResizeRequestBuilder;
use WHMCS\Module\Server\SolusVm2Vps\SolusAPI\Resources\ServerResource;

/**
 * @package WHMCS\Module\Server\SolusVm2Vps\WhmcsAPI
 */
class Product
{
    public const MODULE_NAME = 'solusvm2vps';
    public const PACKAGE_OPTIONS_TYPE = 'package';
    public const CONFIG_OPTIONS_TYPE = 'configoptions';
    public const ENABLED_IN_PRODUCT = 'on';
    public const ADDITIONAL_DISK_NAME = 'Additional disk (WHMCS)';

    public static function updateDomain(int $serviceId, string $domain): void
    {
        localAPI('UpdateClientProduct', [
            'serviceid' => $serviceId,
            'domain' => $domain,
        ]);
    }

    public static function upgrade(int $upgradeId): void
    {
        $upgrade = Upgrade::getById($upgradeId);
        $serviceId = (int) $upgrade->relid;

        if ($upgrade->type !== self::PACKAGE_OPTIONS_TYPE) {
            return;
        }
        list ($newProductId) = explode(',', $upgrade->newvalue);

        $newProduct = ProductModel::getById($newProductId);

        if (empty($newProduct) || $newProduct->servertype !== self::MODULE_NAME) {
            return;
        }
        $newPlanId = (int) $newProduct->configoption1;
        $hosting = Hosting::getByServiceId($serviceId);
        $server = SolusServer::getByServiceId($serviceId);
        $serverId = (int) $hosting->server;
        $serverParams = Server::getParams($serverId);

        $request = [
            'plan_id' => $newPlanId,
            'preserve_disk' => false,
            'backup_settings' => [
                'enabled' => $newProduct->configoption6 === self::ENABLED_IN_PRODUCT,
                'schedule' => [
                    'type' => 'daily',
                    'time' => [
                        'hour' => 0,
                        'minutes' => 0,
                    ],
                ],
            ],
        ];

        $serverResource = new ServerResource(Connector::create($serverParams));
        $serverResource->resize($server->server_id, $request);
    }

    public static function upgradeOption(int $upgradeId): void
    {
        $upgrade = Upgrade::getById($upgradeId);

        if ($upgrade->type !== self::CONFIG_OPTIONS_TYPE) {
            return;
        }

        list ($configOptionId, $currentConfigOptionValueId) = explode('=>', $upgrade->originalvalue);
        list ($newConfigOptionValueId) = explode(',', $upgrade->newvalue);

        $hosting = Hosting::getByServiceId($upgrade->relid);
        $server = SolusServer::getByServiceId($upgrade->relid);
        $product = ProductModel::getById($hosting->packageid);

        $serverResource = new ServerResource(Connector::create(Server::getParams($hosting->server)));

        $configOptionName = ConfigurableOptionsValues::getNameById($configOptionId);
        $newConfigOptionValue = ConfigurableOptionsValues::getValueById($newConfigOptionValueId);

        // Resize
        $emulatedModuleParams = [
            'configoption1' => $product->configoption1,
            "configoptions.$configOptionName" => $newConfigOptionValue,
        ];
        $resizeRequest = ServerResizeRequestBuilder::fromWHMCSUpgradeDowngradeParams($emulatedModuleParams)->get();
        // We have to resize server only for case when we customize plan properties because plan_id isn't change here
        if (isset($resizeRequest['custom_plan'])) {
            $serverResource->resize($server->server_id, $resizeRequest);
        }

        // Additional IPs
        if ($configOptionName === ProductConfigOption::EXTRA_IP_ADDRESS) {
            $additionalIpCount = (int)$newConfigOptionValue;
            $solusServer = $serverResource->get($server->server_id);
            $serverAdditionalIps = array_filter(Arr::get($solusServer, 'data.ip_addresses.ipv4'), function (array $ip) {
                return $ip['is_primary'] === false;
            });
            $serverAdditionalIpCount = count($serverAdditionalIps);
            if ($additionalIpCount > $serverAdditionalIpCount) {
                $needIpCount = $additionalIpCount - $serverAdditionalIpCount;

                $serverResource->createAdditionalIps($server->server_id, $needIpCount);
            } elseif ($additionalIpCount < $serverAdditionalIpCount) {
                // Remove IPs from the end
                $reversedServerAdditionalIps = array_reverse($serverAdditionalIps);
                $reversedServerAdditionalIpsForDelete = array_slice(
                    $reversedServerAdditionalIps,
                    0,
                    $serverAdditionalIpCount - $newConfigOptionValue
                );
                $ipIdsForDelete = array_map(static function (array $additionalIp) {
                    return $additionalIp['id'];
                }, $reversedServerAdditionalIpsForDelete);

                $serverResource->deleteAdditionalIps($server->server_id, $ipIdsForDelete);
            }
        }

        // Additional disk
        if ($configOptionName === ProductConfigOption::ADDITIONAL_DISK) {
            $additionalDiskSize = (int)$newConfigOptionValue;
            $disks = Arr::get($serverResource->getDisks($server->server_id), 'data');
            $additionalDisks = array_values(array_filter($disks, function (array $disk) {
                return $disk['is_primary'] === false && $disk['name'] === Product::ADDITIONAL_DISK_NAME;
            }));

            if (count($additionalDisks) > 1) {
                throw new Exception('Multiple additional disks are not supported.');
            }

            $additionalDisk = $additionalDisks[0] ?? null;

            if (!$additionalDisk && $additionalDiskSize > 0) {
                $additionalDiskOfferId = (int)$product->configoption9;
                if ($additionalDiskOfferId === 0) {
                    throw new Exception('Need to specify an additional disk offer for adding additional disks.');
                }
                $serverResource->createAdditionalDisk($server->server_id, [
                    'name' => Product::ADDITIONAL_DISK_NAME,
                    'size' => $additionalDiskSize,
                    'offer_id' => $additionalDiskOfferId,
                ]);
            }

            if ($additionalDisk && $additionalDiskSize > 0) {
                if ($additionalDiskSize < $additionalDisk['size']) {
                    throw new Exception('Disk size must be greater than current one.');
                }
                if ($additionalDiskSize > $additionalDisk['size']) {
                    $serverResource->resizeAdditionalDisk($server->server_id, $additionalDisk['id'], [
                        'size' => $additionalDiskSize,
                    ]);
                }
            }

            if ($additionalDisk && $additionalDiskSize === 0) {
                $serverResource->deleteAdditionalDisk($server->server_id, $additionalDisk['id']);
            }
        }
    }
}
